
#  ------------------------------------------------------------------------


# Korespondecni analyza - CA-----------------------------------------------


#  ------------------------------------------------------------------------

install.packages(c('vegan', 'gclus', 'cluster', 'vegan', 'ade4','permute', 'lattice'))


library(permute); library(lattice); library(gclus); library(cluster);library(vegan); 


# nastaveni cesty
setwd("C:/Users/ucitel/Desktop/FSTA_2015/09_cviceni")
getwd()
dir()

ryby<-read.table('ryby.csv', sep=";", header=TRUE, dec=".")
fix(ryby)


# Priklad 1. --------------------------------------------------------------
# Provedte koreposndecni analyzu na datech o abundancich ryb. Zjistete, ktere druhy spolu souviseji, ktere lokality
# a kter lokality a druhy

ryby.ca<-cca(ryby)  # vypocet korespondecni ananlyzy

summary(ryby.ca)    # defaultne nastaveno skalovani 2
summary(ryby.ca, scaling=1) 

# Kolik os budeme interpretovat =  do kolika rozmeru si muzeme prostor zjednodusit

# 1) KS kriterium
eig<-ryby.ca$CA$eig

eig[eig > mean(eig)]

# 2) BMS model
e<-eig
n<-length (e)
bsm<-data.frame(j=seq(1:n), p=0)
bsm$p<-1/n
for (i in 2:n) {
  bsm$p[i] = bsm$p[i-1]+ (1/(n+1-i))
}
bsm$p<-100*bsm$p/n
bsm

# Graf valstnich hodnot a % variability pro kazdou hlavni komponentu

par(mfrow=c(2,1))
barplot(eig, main="Vlastni hodnoty", col="bisque", las=2)
abline(h=mean(eig), col="red")
legend ("topright", "Prumerna vlastni hodnota", lwd=1, col=2, bty = "n")

barplot (t(cbind(100*eig/sum(eig),bsm$p[n:1])), beside=TRUE, mail="% variability", col=(c("bisque",2)), las=2)
legend ("topright", c("% variability","Broken stick model"), pch=15, col=(c("bisque",2)), bty = "n")


#  ------------------------------------------------------------------------

evplot(eig)   # graf vlastnich hodnot

# CA graf (biplot)

par(mfrow=c(1,2))
plot(ryby.ca, scaling=1, main="CA na abundancich ryb \n skalovani 1")
# radky jsou centroidy sloupcu;  ordinace objektu; vicerozmerny prostor zachovava chi-square vzdalesnot mezi objekty
# 1) Vzdalenost mezi objekty aproximuje jejich puvodni chi-square vzdalesnot -> objekty, kter si jsou blizke, 
# jsou relativne podobne dle vyskytu druhu
# 2) Blizkost objektu a druhu predstavuje, ze dany objekt obsahuje vysoky pocet jedincu daneho druhu 

plot(ryby.ca, scaling=2, main="CA na abundancich ryb \n skalovani 2")
# sloupce jsou centroidy radku;  ordinace druhu; vicerozmerny prostor zachovava chi-square vzdalesnot mezi druhy
# 1) Vzdalenost mezi druhy aproximuje jejich puvodni chi-square vzdalenost -> druhy, kter si jsou blizke, 
# se vyskytuji na stejnych lokalitach
# 2) Blizkost objektu a druhu predstavuje, ze dany druh se hojne vyskytuje na danych lokalitach 


# Vysvetleni os pomoci ----------------------------------------------------
# enviromentalnich promennych ---------------------------------------------

lokality <-read.table('lokality.csv', sep=";", header=TRUE, dec=".")

# Pouzijeme fuknci envfit(), ktera dodatecne vykresli environmentalni promenne do CA  

ryby.ca.env<-envfit(ryby.ca,lokality )

# Vykreslime graf -> je potreba aby byl aktivni biplot se skalovani 2 
par(mfrow=c(1,1))
plot(ryby.ca, scaling=2, main="CA na abundancich ryb \n skalovani 2")
plot(ryby.ca.env) # Post-hoc interpretace ordinacnich os



# Priklad 2 ---------------------------------------------------------------

# Provedte CA na datovem souboru wine.csv, vynechte posledni promennou (predstavuje celkovy soucet bodu)

wine.puv <-read.table('wine.csv', sep=";", header=TRUE, dec=".")
fix(wine.puv)

wine <-t(wine.puv[,2:ncol(wine.puv)]) # pro snadnejsi interpretci si data transponujeme

colnames(wine)<-wine.puv[,1]          # priradime nazvy sloupcu

wine.ca<-cca(wine[1:(nrow(wine)-1),]) # vypocet CA, 1. sloupec jsou popisky a posledni vynechame
  
wine.eig<-wine.ca$CA$eig              # vlastni hodnoty
summary(wine.ca)

evplot(wine.eig)                      # prostor se da zjednodusit do 3 os 

par(mfrow=c(1,2))
plot(wine.ca, scaling=1, main="CA wine.csv \n skalovani 1")
plot(wine.ca, scaling=2, main="CA wine.csv \n skalovani 2")


# Ukol 1 ------------------------------------------------------------------

# Provedte CA na datech spisovatele.csv a zjistet zda je nektere pismeno typicke pro nektereho spisovatele. 

spi <-read.table('spisovatele.csv', sep=";", header=TRUE, dec=".")

plot(cca(spi[,-1]), scaling=2)

spi
#  ------------------------------------------------------------------------

#  ------------------------------------------------------------------------


# Analyza hlavnich koordinat - PCoA ---------------------------------------



# Priklad3  ---------------------------------------------------------------

# Provedte PCoA na datech ryby.csv, pouzijte Bray-Curtis index

ryby.bray<-vegdist(ryby, method="bray")

ryby.bray.pcoa<-cmdscale(ryby.bray, k=(nrow(ryby)-1), eig=TRUE) # vypocet PCoA -> je nutne nastavit kolik dimenzi ma spocitat

# Graf objektu a posteriorni projekci druhu jako vazenych prumeru
par(mfrow=c(1,1))
ordiplot(scores(ryby.bray.pcoa)[,c(1,2)], type="t", main="PCoA - druhy")
abline(h=0, lty=3)
abline(v=0, lty=3)

# pridame druhy
ryby.wa<-wascores(ryby.bray.pcoa$points[,1:12], ryby)
text(ryby.wa, rownames(ryby.wa), cex=0.7, col="red") # interpretace grafu je stejna jako v pripade CA



#  ------------------------------------------------------------------------

# Nemetricke skalovani - NMDS ---------------------------------------------


# Priklad 4 ---------------------------------------------------------------

# Provedte nemetricke vicerozmerne skalovani na datech ryby

# pouzijeme funkci metaMDS(ryby, distance="bray)

ryby.nmds<-metaMDS(ryby, distance="bray")
ryby.nmds   # vysledek

ryby.nmds$stress

plot(ryby.nmds, type="t", main =paste("NMDS/ Bray - Stress=",round(ryby.nmds$stress,3))) # interpretace stejna jako u CA

# nyni je potreba zhodnotit zda NMDSdobre odpovida nasim datum -> Sheparduv diagram a graf shody

par(mfrow=c(1,2))
stressplot(ryby.nmds, main="Sheparduv diagram")
gof=goodness(ryby.nmds)
plot(ryby.nmds, type="t", main="Graf shody")
points(ryby.nmds, display="sites", cex=gof*100)
