
#  ------------------------------------------------------------------------

# Diskriminacni analyza ---------------------------------------------------

#  ------------------------------------------------------------------------


install.packages(c("klaR","permute","lattice", "ellipse","MASS","vegan"))
library(permute); library(lattice);
library(vegan);library(MASS); library(ellipse); 
library(klaR)

setwd("g:/FSTA_2015/10_cviceni")
getwd()


lokality<-read.table("lokality.csv", sep=";", dec=".", header=TRUE)
fix(lokality)
ryby<-read.table("ryby.csv", sep=";", dec=".", header=TRUE)

names(lokality)




# Priklad 1:
# Provedte diskriminacni analyzu na datovem souboru lokality
# 1) Zjistete diskriminacni schopnosti promennych a vyberte ty, ktere budou vstupovat do vypoctu
# 2) Overte homogenitu vnitroskupinove kovariacni matice
# 3) Zjistete diskriminacni schopnost celeho modelu
# 4) Vytvorte model a popiste jej

# -> 1) Zjistete diskriminacni schopnosti promennych a vyberte ty, ktere budou vstupovat do vypoctu
# Pomoci funkce greedy.wilks() v balicku "klaR"

gr<-cutree(hclust(vegdist(ryby, "jac", binary=T), method="ward.D"), k=4)  # potreba vytvorit kategorialni promennou 
lokality2<-cbind(lokality[,2:ncol(lokality)], gr)                         # spojeni kategorialni promenne a dat

?greedy.wilks
lokality.step<- greedy.wilks(gr ~ ., data=lokality2, niveau = 0.1)  # vypocet modelu pomoci dopredne selekce 
lokality.step                                                       # model postaveny pomoci dopredne selekce 

lok.vyb<-lokality[,c("alt","dbo","oxy")]

# -> 2) Overte homogenitu vnitroskupinove kovariacni matice
# Overeni vicerozmerne homogenity vnitroskupinove kovariacni matice pomoci funkce betadisper() 
# -> jedna se o vicerozmernou obdobu Leveneho testu homogenity rozptylu

lok.vyb.dist<-dist(lok.vyb)
lok.MHV<-betadisper(lok.vyb.dist, gr)
anova(lok.MHV)
permutest(lok.MHV)
# vnitroskupinova kovariacni matice JE homogenni -> neni potreba provadet transformaci

# -> 3) Zjistete diskriminacni schopnost celeho modelu
# Zjistime Wilks lamba pro cely model pomoci funkce manova()

summary(manova (cbind(lok.vyb$alt,lok.vyb$oxy,lok.vyb$dbo) ~ gr), test="Wilks") # Wilks Lamba  je nizke -> model dobre predikuje


# -> 4) Vytvorte model a popiste jej
# funkce lda()
?lda 
lokality.lda<-lda (gr ~ alt + oxy + dbo, data=lok.vyb)  # vypocet diskriminacni analyzy
lokality.lda

lokality.lda$prior  # apriorni pravdepodobnost
lokality.lda$means  # prumery pro dane tri promenne pro 4 skupiny

cs<-lokality.lda$scaling  # normalizovane vektory -> standardizovane koeficienty pro diskriminacni funkce
lokality.lda$svd^2        # kanonicke vlastni hodnoty
fp<-predict(lokality.lda)$x   # pozice objektu v prostoru kanonickych os

lokality.class<-predict(lokality.lda)$class     # klasifikace objektu
lokality.class
lokality.post<-predict(lokality.lda)$posterior  # posteriorni pravdepodobnost pro obejkt, ze patri do dane skupiny 
lokality.post

lokality.table<-table(gr, lokality.class)
lokality.table

diag(prop.table(lokality.table,1))   # procento spravne zaklasifikovanych  pripadu pro jednotlive skupiny

# Graf pro objekty ve vicerozmernem prostoru kanonickych os obarvenzch podle prislusnosti objektu do skupiny

par(mfrow=c(1,2))
plot(fp[,1], fp[,2], type="n")
text(fp[,1], fp[,2], rownames(lokality), col=c(as.numeric(lokality.class)+1))   # "nova" klasifikace
abline(h=0, lty="dotted")
abline(v=0, lty="dotted")

# vykresleni elips kolem skupin bodu 
for (i in 1:length(levels(as.factor(lokality.class)))){
  cov <- cov(fp[lokality.class==i,])
  centre <- apply (fp[lokality.class==i,],2,mean)
  lines(ellipse(cov, centre=centre, level=0.95))
}

plot(fp[,1], fp[,2], type="n")
text(fp[,1], fp[,2], rownames(lokality), col=c(as.numeric(gr)+1))   # puvodni klasifikace
abline(h=0, lty="dotted")
abline(v=0, lty="dotted")

# vykresleni elips kolem skupin bodu 
for (i in 1:length(levels(as.factor(gr)))){
  cov <- cov(fp[gr==i,])
  centre <- apply (fp[gr==i,],2,mean)
  lines(ellipse(cov, centre=centre, level=0.95))
}


# Priklad 2:
# Dle vytvoreneho modelu zaklasifikujte novou lokalitu s parametry alt=830, oxy=94 a dbo=42

new.lok<-c(830,94,42) # vytvorime novy objekt
new.lok<-as.data.frame(t(new.lok))  # musi to byt radek v tabulce
colnames(new.lok)<-colnames(lok.vyb) # pojmenujeme promenne

predict.new.lok<-predict(lokality.lda, newdata=new.lok) # zaklasifikuje dany objekt
predict.new.lok

# Ukol 1
# Vytvorte predikcni model na datech banloan.csv a nasledne zaklasifikujte pripady,ktere nejsou kasifikovany.

bank<-read.table("bankloan.csv", sep=";", dec=".", header=TRUE)
names(bank)

summary(bank$Previously.defaulted)

bank.hotovo<-bank[!is.na(bank$Previously.defaulted),]
bank.class<-bank[is.na(bank$Previously.defaulted),]

banka.step<- greedy.wilks(Previously.defaulted ~ ., data=bank.hotovo, niveau = 0.1)

banka.vyb<-bank.hotovo[,c("Years.with.current.employer","Credit.card.debt.in.thousands", "Years.at.current.address",
             "Other.debt.in.thousands","Household.income.in.thousands","Previously.defaulted")]

bank.vyb.dist<-dist(banka.vyb[,-6])
lok.MHV<-betadisper(bank.vyb.dist, banka.vyb[,6])
anova(lok.MHV)
permutest(lok.MHV)

banka.lda<-lda (Previously.defaulted ~ Years.with.current.employer+Credit.card.debt.in.thousands+Years.at.current.address+
             Other.debt.in.thousands+Household.income.in.thousands, data=banka.vyb)

banka.lda

banka.class<-predict(banka.lda)$class     # klasifikace objektu
banka.class

lokality.table<-table(banka.vyb$Previously.defaulted, banka.class)
lokality.table

(500+62)/(500+62+121+17)

bank.class$bank.class.fin<-predict(banka.lda, newdata=bank.class)$class
fix(bank.class)
